QuestAnnouncer4 = LibStub("AceAddon-3.0"):NewAddon("QuestAnnouncer4", "AceConsole-3.0", "AceComm-3.0", "AceEvent-3.0", "AceSerializer-3.0", "AceTimer-3.0");

local _, _, rev = string.find("$Rev: 31 $", "([0-9]+)");
QuestAnnouncer4.version = "2.2 r"..rev;
QuestAnnouncer4.author = "Jason Olivarez (Svetlania, Ragu, and Pyroclastic of Garrosh-US)";

-- Local variables
local questLog = {};
local questLogUpdating = false;
local delayPartyChat = false;

-- Default options DB
local defaults = {
	global = {
		PartyChat = false,
		localMsgs = true,
	}
};

-- Message List
local messages = {
	-- quest failed messages
	["failed"] = {
		color = {r=1,g=0,b=0},
		broadcast = "%s has failed %s",
		party = "Failed: %s",
		locl = "You have failed %s"
	},
	-- quest progress messages
	["progress"] = {
		color = {r=1,g=1,b=0},
		broadcast = "%s\'s quest progress... %s",
		party = "%s",
		locl = nil
	},
	-- quest completed messages
	["completed"] = {
		color = {r=0,g=1,b=0},
		broadcast = "%s has completed %s",
		party = "Completed: %s",
		locl = "You have completed %s"
	},
	-- quest accepted messages
	["accepted"] = {
		color = {r=.75,g=.75,b=1},
		broadcast = "%s has accepted %s",
		party = "Accepted: %s",
		locl = "You have accepted %s"
	}
}

function QuestAnnouncer4:OnInitialize()
	-- Load Options DB
	self.db = LibStub("AceDB-3.0"):New("QuestAnnouncer4DB", defaults);
	
	-- Initialize Config
	self:InitConfig();
	
	-- Register Events
	self:RegisterEvent("QUEST_LOG_UPDATE");
	self:RegisterEvent("QUEST_ACCEPTED");
	self:RegisterEvent("UI_INFO_MESSAGE");
	
	-- Register Communications channel
	self:RegisterComm("QA4");
	
	-- Delay welcome message.
	self:ScheduleTimer(function() 
			self:Print("QuestAnnouncer 4 (ver. " .. self.version .. ") is loaded. Use /qa4 for options.")
		end, 4);
end

function QuestAnnouncer4:OnCommReceived(prefix, message, distribution, sender)
	if (prefix == "QA4") and (sender ~= UnitName("player")) then
		local success, msg, color = self:Deserialize(message)
		if (success) then
			UIErrorsFrame:AddMessage(msg,color.r,color.g,color.b,1.0,UIERRORS_HOLD_TIME);
		end
	end
end

function QuestAnnouncer4:ProcessAnnouncement(typ, msg)
	local myname = UnitName("player");
	local broadcast, party, locl, color = messages[typ].broadcast, messages[typ].party, messages[typ].locl, messages[typ].color
	if locl and self.db.global.localMsgs then
		UIErrorsFrame:AddMessage(string.format(locl, msg),color.r,color.g,color.b,1.0,UIERRORS_HOLD_TIME);
	end

	if (GetNumSubgroupMembers(LE_PARTY_CATEGORY_HOME) > 0) then
		local serializedData = self:Serialize(string.format(broadcast, myname, msg), color);
		self:SendCommMessage("QA4", serializedData, "RAID");
		
		if (not delayPartyChat or typ ~= "progress") and self.db.global.PartyChat then
			if typ == "progress" then
				delayPartyChat = true;
				self:ScheduleTimer(function() delayPartyChat = false end, 3);
			end
			SendChatMessage(string.format(party, msg), "PARTY");
		end
	end
end

function QuestAnnouncer4:UI_INFO_MESSAGE(frame,msgtype,msg)
	local validmsgtype = {
		[241] = false, -- trade canceled
		[298] = false, -- new location discovered
		[302] = true, -- object complete
		[303] = true, -- objective complete
		[304] = true, -- quest progress (kill)
		[305] = true, -- quest progress
		[306] = true, -- quest progress (item looted)
		[390] = false, -- discovered new territory
		[395] = false, -- no fish are hooked
		[448] = false, -- mail sent
		[800] = false, -- removed from queue for timeout
		[803] = false, -- role check has been initiated
		[807] = false, -- now queue in dungeon
		[814] = false, -- you are no longer queued
	}
	if not QuestAnnouncer4DB.messages then QuestAnnouncer4DB.messages = {} end
	
	if (validmsgtype[msgtype] ~= nil) then
		if (validmsgtype[msgtype] == true) then
			self:ProcessAnnouncement("progress", msg);
		end
	else
		self:Print("UI_INFO_MESSAGE: "..msgtype..": "..msg)
		table.insert(QuestAnnouncer4DB.messages, (msgtype..": "..msg))
		
	end
end

function QuestAnnouncer4:QUEST_ACCEPTED(event, QuestID)
	if ( QuestID > 0 ) then
		local questTitle = C_QuestLog.GetTitleForQuestID(QuestID);
		self:ProcessAnnouncement("accepted", questTitle)
	else
		self:Print("Detected a quest with id 0: "..QuestID)
	end
end

function QuestAnnouncer4:QUEST_LOG_UPDATE()
	-- Check to see if thread is already running. If so, exit or lock if not.
	if ( questLogUpdating ) then
		return true;
	end
	questLogUpdating = true;
	
	local numEntries, numQuests = C_QuestLog.GetNumQuestLogEntries();
	for i = 1,numEntries do
		local info = C_QuestLog.GetInfo(i)
		if not info.isHeader and info.title and info.questID then
			local status = 0
			local prev_status = questLog[info.questID]
			if (C_QuestLog.IsComplete(info.questID)) then status = 1 end
			if (C_QuestLog.IsFailed(info.questID)) then status = -1 end
			
			-- Check for Quest changes
			if prev_status and prev_status ~= status then
				local sendDelayed = function(typ, msg)
					self:ScheduleTimer(function() self:ProcessAnnouncement(typ, msg) end, 1)
				end
				if prev_status == 0 and status == 1 then
					sendDelayed("completed", info.title);
				elseif (prev_status == 0 or prev_status == 1) and status == -1 then
					sendDelayed("failed", info.title);						
				end
			end
			questLog[info.questID] = status;
		end
	end
	
	-- Unlock the thread
	questLogUpdating = false;
end